#include <iostream>
#include <vector>
#include <sstream>
#include "GStart.hpp"
#include "Grammar.hpp"
#include "CNGGGPS.hpp"
#include "Individual.hpp"
#include "Result.hpp"
#include "Context.hpp"
#include "Evaluator.hpp"
#include "SymRegEvaluator.hpp"
#include "Population.hpp"
#include "Log.hpp"

using namespace std;

CNGGGPS::CNGGGPS(string strName, int iRuns, int iPopSize, int iGenerations, double dMutationRate, double dCrossoverRate, int iMinDepth, int iMaxDepth, int iTournamentSize, int iTries)
{
	Log* plog = new Log(strName, iRuns, iGenerations);
	Evaluator* peval = new Evaluator();
	Context* pcontext = new Context();
	Population* ppop;
	Population* ppopNew;
	vector<Individual*> vpindBestOfRun;
	vpindBestOfRun.reserve(iRuns);

	plog->logParameters(iPopSize,dCrossoverRate,dMutationRate,iTries,iMinDepth,iMaxDepth,iTournamentSize); 
	for(int iRun = 0; iRun < iRuns; iRun++)
	{
		ppop = new Population(pcontext, peval, iPopSize, iTournamentSize, iMinDepth, iMaxDepth);
		ppop->generateInitialPopulation();
		
		vpindBestOfRun.push_back(ppop->getBestIndividual());
		plog->addEntry(iRun, 0, ppop->getBestFitness(), ppop->getAverageFitness(), ppop->getBestHits(), ppop->getAverageHits()); 


		for(int iGen = 1; (iGen < iGenerations) && (ppop->getBestFitness() < 1); iGen++)
		{
			cout << "Beginning generation " << iGen << endl;
			ppopNew = new Population(pcontext, peval, iPopSize, iTournamentSize, iMinDepth, iMaxDepth);
			
			for(int j = 0; j < iPopSize; j++)
			{
				ppopNew->addIndividual(ppop->selectIndividual());
			}
			delete ppop;
			ppop = ppopNew;

	//cout << "Mutate" << endl;
			ppop->mutate(dMutationRate);
	//cout << "Crossover" << endl;
			ppop->crossover(dCrossoverRate);
	//cout << "Evaluate" << endl;
			ppop->evaluate();

			if(ppop->getBestFitness() > vpindBestOfRun.at(iRun)->getFitness())
			{
				delete vpindBestOfRun.at(iRun);
				vpindBestOfRun.at(iRun) = ppop->getBestIndividual();
			}
			plog->addEntry(iRun, iGen, ppop->getBestFitness(), ppop->getAverageFitness(), ppop->getBestHits(), ppop->getAverageHits()); 
		}
		delete ppop;
	}
//	cout << "Avg Fit: " << ppop->getAverageFitness() << endl;
//	cout << "Best Ind: " << ppop->getBestIndividual()->toString() << endl;

	plog->logBest(vpindBestOfRun);

	for(vector<Individual*>::iterator it = vpindBestOfRun.begin(); it!=vpindBestOfRun.end(); it++)
	{
		cout << (**it).toString() << endl;
		delete (*it);
	}
	delete peval;
	delete pcontext;
	delete plog;
}

CNGGGPS::~CNGGGPS()
{
}

double atod(string str);

int main(int argc, char* argv[])
{
	int iGenerations=50;
	int iRuns=10;
	int iPopulationSize=200;
	double dMutationRate=0.2;
	double dCrossoverRate=0.8;
	int iMinDepth=1;
	int iMaxDepth=30;
	int iTournamentSize=3;
	int iTries=5;
	string strName="expr";
	srand(time(NULL));

//cout << "Start" << endl;
	for(int i = 0; i < argc; i++)
	{
		if(argv[i][0] == '-')
		{
			switch(argv[i][1])
			{
			case 'g':
				iGenerations = atoi(argv[i+1]);
				break;
			case 'r':
				iRuns = atoi(argv[i+1]);
				break;
			case 'p':
				iPopulationSize = atoi(argv[i+1]);
				break;
			case 'f':
			case 'n':
				strName = string(argv[i+1]);
				break;
			case 'c':
				dCrossoverRate = atod(argv[i+1]);
				break;
			case 'm':
				dMutationRate = atod(argv[i+1]);
				break;
			case 'i':
				iMinDepth = atoi(argv[i+1]);
				break;
			case 'x':
				iMaxDepth = atoi(argv[i+1]);
				break;
			case 't':
				iTournamentSize = atoi(argv[i+1]);
				break;
			case 'y':
				iTries = atoi(argv[i+1]);
				break;
			case 's':
				srand(atoi(argv[i+1]));
				break;
			case 'h':
			case '?':
				cout << "usage: cngggps [options]" << endl;
				cout << endl;
				cout << "common user options, with defaults in [ ]:" << endl;
				cout << "-h -?\t\tshow this message" << endl;
				cout << "-n <prefix>\tset logfile prefix [expr]" << endl;
				cout << "-r <value>\tset number of runs [5]" << endl;
				cout << "-g <value>\tset maximum generations [50]" << endl;
				cout << "-p <value>\tset population size [100]" << endl;
				cout << "-c <value>\tset crossover rate [0.8]" << endl;
				cout << "-m <value>\tset mutation rate [0.2]" << endl;
				cout << "-i <value>\tset minimum symbol tree depth [1]" << endl;
				cout << "-x <value>\tset maximum symbol tree depth [30]" << endl;
				cout << "-t <value>\tset tournament size [3]" << endl;
				cout << "-y <value>\tset crossover/mutation attempts [5]" << endl;
				cout << "-s <value>\tset random seed [time]" << endl;
				cout << endl;
				cout << "Creativly Named Grammar Guided Genetic Programming System" << endl;
				cout << "Copyright 2006 - Stephen E. Baker" << endl;
				return 0;

			}
		}
	}
	delete (new CNGGGPS(strName,iRuns,iPopulationSize,iGenerations,dMutationRate,dCrossoverRate,iMinDepth,iMaxDepth,iTournamentSize,iTries));

//cout << "End" << endl;
	return 0;
}

double atod(string str)
{
	stringstream ss;
	ss << str;
	double d;
	ss >> d;
	return d;
}
