#ifndef CNGGGPS_EVALUATOR
#define CNGGGPS_EVALUATOR

#include "Individual.hpp"

/**
 * \brief Evaluates individuals against criteria.
 * \author Stephen E. Baker
 * \date 2006-2007
 * \version 1.0
 *
 * Calculates the fitness of individuals by setting up the context, running
 * them and comparing the result to the expected value.  At least this class
 * must be modified or extended for any new problem.
 */
class Evaluator
{
public:
	/**
	 * Creates and intializes evaluator object.  If values need to be
	 * loaded from a file or any precalculation needs to be done it should
	 * be done here.  This method is only executed once, whereas evaluate
	 * is run thousands of times depending on runtime parameters.
	 */
	Evaluator();

	/**
	 * Evaluates an individual to determine it's fitness.  Typically this 
	 * method would setup the context, execute the individual with it, 
	 * compare the result to the desired result, transform this into a
	 * fitness value between 0 and 1 where 1 is the highest, and associate
	 * this fitness value with the individual.  It is recommended to
	 * read the included template and follow it closely.
	 *
	 * \param[in,out] ind
	 * 	The individual or program to be evaluated
	 * \param[in,out] context
	 * 	The environment and argument values the program is to be
	 * 	run with
	 * \return
	 * 	The fitness value
	 * \warning
	 * 	If the GP is keeping track of hits, they must be added to the
	 * 	individual using setHits
	 * \sa
	 * 	Individual::setHits
	 * \sa
	 * 	Context::setArgumentValue
	 * \sa
	 * 	Context::clear
	 * \sa
	 * 	Individual::execute
	 * \sa
	 * 	Result
	 */
	virtual double evaluate(Individual &ind, Context &context);

	virtual	~Evaluator();
};

#endif

