#ifndef CNGGGPS_ARGUMENT
#define CNGGGPS_ARGUMENT

#include <string>
#include "Function.hpp"
#include "Context.hpp"
#include "Result.hpp"

/**
 * \brief Function which returns an argument value.
 * \author Stephen E. Baker
 * \date 2006-2007
 * \version 1.0
 */
class FArgument: public Function
{
public:
	/**
	 * Creates a new argument with the specified key
	 *
	 * \param[in] strName
	 * 	The key or formal parameter associated with this symbol
	 */
	FArgument(std::string strName);

	/**
 	 * Looks up the key in the context corresponding to itself and returns 
	 * the value associated with it.
	 *
	 * \param[in] context
	 * 	A mapping of keys to values.
	 * \return
	 * 	The value of the symbol as specified by the context.
	 * \sa
	 * 	Result
	 * \sa
	 * 	Context::setArgumentValue
	 * \sa
	 * 	Context::getArgumentValue
	 */
	Result* execute(Context &context);

	/**
	 * \sa
	 * 	Grammar::clone
	 */
	Grammar* clone();

	~FArgument();
private:
	/**
	 * Key associated with the object
	 */
	std::string strName;
};

#endif

