#ifndef CNGGGPS_GRAMMAR_CONST
#define CNGGGPS_GRAMMAR_CONST

#include <cmath>
#include "Function.hpp"

/**
 * \brief Ephemeral Constant
 * \author Stephen E. Baker
 * \date 2006-2007
 * \version 1.0
 *
 * Acts as a constant or literal in the program.  This constant may be specified
 * in GrammarRules or left to be randomly generated when the object is
 * instantiated.
 */
class FConst: public Function
{
public:
	/**
	 * Creates a new function constant with the specified value.  If
	 * none is given a random value between 0 and 100 is used.
	 *
	 * \param[in] dConst
	 * 	The constant value this function should have.
	 */
	FConst(double dConst = HUGE_VAL);

	/**
	 * Returns the value of this constant.
	 * 
	 * \param[in] context
	 * 	Unused by this function but left for consistancy.
	 * \sa
	 * 	Result
	 * \sa
	 * 	Grammar::execute
	 */
	Result* execute(Context &context);

	/**
	 * \sa
	 * 	Grammar::clone
	 */
	Grammar* clone();
	
	~FConst();
private:
	/**
	 * The value associated with this constant
	 */
	double dConst;
	
	/**
	 * Generates a random value between 0 and 100 to be associated with
	 * this constant if no value is given.
	 */
	void generateValue();
};

#endif

