#include <string>
#include <vector>
#include "Individual.hpp"
#include "GrammarRules.hpp"
#include "GStart.hpp"
#include <iostream>

using namespace std;

Individual::Individual()
{
	pgramRoot = NULL;
}

Individual::Individual(Context* pcontext, bool bInitialize, int iMinDepth, int iMaxDepth)
{
	this->pcontext = pcontext;
	
	if(bInitialize)
	{
		pgramRoot = new GStart();
		iDepth = 1;
		this->iMinDepth = iMinDepth;
		this->iMaxDepth = iMaxDepth;

		buildTree(pgramRoot, 0);
	}
	else
	{
		pgramRoot = NULL;
	}
}

Grammar* Individual::getRandomNode() const
{
	int iRnd;

	//Don't choose the start node unless it is the only one available, would
	//either replace the entire tree or swap one program with another
	if(vpgramNodeList.size() > 1)
		iRnd = (rand()%(vpgramNodeList.size()-1))+1;
	else
		iRnd = 0;

	return vpgramNodeList.at(iRnd);
}

Grammar* Individual::getRandomNodeMatching(string strGrammarID) const
{
	int iRnd;
	vector<Grammar*> vpgramMatchingNodes;

	for(vector<Grammar*>::const_iterator it = vpgramNodeList.begin(); it != vpgramNodeList.end(); it++)
	{
		if(strGrammarID == ((*it)->getGrammarID()))
		{
			vpgramMatchingNodes.push_back(*it);
		}
	}

	if(vpgramMatchingNodes.empty())
		return NULL;

	iRnd = rand()%vpgramMatchingNodes.size();
	return vpgramMatchingNodes.at(iRnd);
}

double Individual::getFitness() const
{
	return dFitness;
}

void Individual::setFitness(double dFitness)
{
	this->dFitness = dFitness;
}

int Individual::getHits() const
{
	return iHits;
}

void Individual::setHits(int iHits)
{
	this->iHits = iHits;
}

int Individual::getMinDepth() const
{
	return iMinDepth;
}

int Individual::getMaxDepth() const
{
	return iMaxDepth;
}

string Individual::toString() const
{
	return (pgramRoot->toString(*pcontext));
}

bool Individual::buildTree(Grammar* pgramNode, int iTries = 0)
{
	int iTry = 0;
	bool bGood = true;
	do
	{
		bGood = addNodeToTree(pgramNode, pgramNode->getDepth());
		if(bGood && iDepth > iMinDepth)
			return true;
		iTry++;
		pgramNode->deleteExpression();
	}while(iTry != iTries);

	return false;
}

void Individual::formNodeList()
{
	vpgramNodeList.clear();
	addSubtreeToList(pgramRoot);
}

Result* Individual::execute(Context &context)
{
	return pgramRoot->execute(context);
}

Individual* Individual::clone() const
{
	Individual* pindClone = new Individual(pcontext, false);
	pindClone->iDepth = iDepth;
	pindClone->iMinDepth = iMinDepth;
	pindClone->iMaxDepth = iMaxDepth;
	pindClone->iHits = iHits;
	pindClone->dFitness = dFitness;
	pindClone->pgramRoot = pgramRoot->clone();
	pindClone->formNodeList();

	return pindClone;
}

Individual::~Individual()
{
	if(pgramRoot != NULL)
		delete pgramRoot;
}

void Individual::addSubtreeToList(Grammar* pgramNode)
{
	if(pgramNode->getNumArguments() == 0)
		return;
	
	vpgramNodeList.push_back(pgramNode);
	for(int i=0; i < pgramNode->getNumArguments(); i++)
	{
		addSubtreeToList(pgramNode->getArgument(i));
	}
}

bool Individual::addNodeToTree(Grammar* pgramNode, int iDepth)
{
	int iRule;
	vector<Grammar*>* pvpgramExpression = NULL;
	
	if(iDepth > iMaxDepth)
	{
		return false;
	}
	if(iDepth > this->iDepth)
	{
		this->iDepth = iDepth;
	}
	pgramNode->setDepth(iDepth);

	pvpgramExpression = new vector<Grammar*>;
	iRule = GrammarRules::getExpressionFor(pgramNode->getGrammarID(), GrammarRules::ANY, pvpgramExpression);\

	if(iRule == -1)
	{
		delete pvpgramExpression;
		pvpgramExpression = NULL;
		return true;
	}

	for(vector<Grammar*>::iterator it = pvpgramExpression->begin(); it != pvpgramExpression->end(); it++)
	{
		if(addNodeToTree(*it,iDepth+1) == false)
		{
			for(vector<Grammar*>::iterator it2 = pvpgramExpression->begin(); it2 != pvpgramExpression->end(); it2++)
			{
				delete (*it2);
			}
			delete pvpgramExpression;

			pvpgramExpression = NULL;
			pgramNode->setExpression(NULL,-1);
			return false;
		}
	}
	pgramNode->setExpression(pvpgramExpression, iRule);
	return true;
}

