#include <iostream>
#include "Population.hpp"
#include "Mutate.hpp"
#include "Crossover.hpp"

using namespace std;

Population::Population(Context* pcontext, Evaluator* peval, int iPopulationSize, int iTournamentSize, int iMinDepth, int iMaxDepth, int iTries)
{
	this->iTries = iTries;
	this->iMinDepth = iMinDepth;
	this->iMaxDepth = iMaxDepth;
	this->iTournamentSize = iTournamentSize;
	this->iPopulationSize = iPopulationSize;
	this->peval = peval;
	this->pcontext = pcontext;

	pvpindPopulation = new vector<Individual*>;
	pvpindPopulation->reserve(iPopulationSize);
}

void Population::generateInitialPopulation()
{
	pvpindPopulation->resize(iPopulationSize);

	for(vector<Individual*>::iterator it = pvpindPopulation->begin(); it != pvpindPopulation->end(); it++)
	{
		*it = new Individual(pcontext, true, iMinDepth, iMaxDepth);
	}

	evaluate();
}

Individual* Population::getBestIndividual() const
{
	return pindBest->clone();
}

Individual* Population::selectIndividual() const
{
	int iRnd;
	int iBest = rand()%iPopulationSize;

	for(int i = 1; i < iTournamentSize; i++)
	{
		iRnd = rand()%iPopulationSize;

		if(((pvpindPopulation->at(iRnd))->getFitness()) > ((pvpindPopulation->at(iBest))->getFitness()))
		{
			iBest = iRnd;
		}
	}
	
	return (pvpindPopulation->at(iBest))->clone();
}

void Population::addIndividual(Individual* ind)
{
	pvpindPopulation->push_back(ind);
}

double Population::getBestFitness() const
{
	return pindBest->getFitness();
}

int Population::getBestHits() const
{
	return pindBest->getHits();
}

double Population::getAverageFitness() const
{
	return dAvgFitness;
}

double Population::getAverageHits() const
{
	return dAvgHits;
}

void Population::mutate(double dMutationRate)
{
	double dRnd;
	Mutate mut (iTries);

	for(vector<Individual*>::iterator it = pvpindPopulation->begin(); it != pvpindPopulation->end(); it++)
	{
		dRnd = double(rand())/RAND_MAX;

		if(dRnd < dMutationRate)
		{
			mut.mutate(**it);
		}
	}
}

void Population::crossover(double dCrossoverRate)
{
	double dRnd;
	Crossover crs (iTries);
	Individual* p1;

	for(vector<Individual*>::iterator it = pvpindPopulation->begin(); it != pvpindPopulation->end(); it++)
	{
		dRnd = double(rand())/RAND_MAX;
		p1 = (*it);
		it++;
	if(it == pvpindPopulation->end()) break; //prevents going past end if odd population size
		if(dRnd < dCrossoverRate)	
		{
			crs.crossover(*p1, **it);
		}
	}
}

void Population::evaluate()
{
	double dTotalFitness = 0;
	double dFitness;
	double dTotalHits = 0;
	double dBestFitness = 0;

	for(vector<Individual*>::iterator it = pvpindPopulation->begin(); it != pvpindPopulation->end(); it++)
	{
		dFitness = peval->evaluate(**it, *pcontext);
		(*it)->setFitness(dFitness);

		dTotalFitness+=dFitness;
		dTotalHits+=((*it)->getHits());

		if((*it)->getFitness() > dBestFitness)
		{
			dBestFitness = (*it)->getFitness();
			pindBest = (*it);	
		}
	}

	dAvgFitness = dTotalFitness/iPopulationSize;
	dAvgHits = dTotalHits/iPopulationSize;
}

Population::~Population()
{
	for(vector<Individual*>::iterator it = pvpindPopulation->begin(); it != pvpindPopulation->end(); it++)
	{
		delete (*it);
	}

	delete pvpindPopulation;
}

